#include "Dealer.h"
#include <iostream>

// Constructor
Dealer::Dealer(std::string name, int age) : Dealer(name, age, true) {}
Dealer::Dealer(std::string name, int age, bool license) : Dealer(name, age, license, "") {}
Dealer::Dealer(std::string name, int age, bool license, std::string homecity) : Human(name, age, homecity) {
	m_license = license;
	m_hand = { nullptr,nullptr };
}

// Copy constructor
Dealer::Dealer(const Dealer& dealer): Human(dealer) {
	m_license = dealer.m_license;
	m_hand = { nullptr,nullptr };
}

// Destructor
Dealer::~Dealer() {
	std::cout << "dealer destroyed!" << std::endl;
}

// Getter for the license of the dealer
bool Dealer::GetLicense() const { return m_license; }

// Set the license as false
void Dealer::InvokeLicense() {
	m_license = false;
}

/// <summary>
/// Checks if the dealer has a blackjack (21) in hand
/// </summary>
/// <returns></returns>
bool Dealer::IsBlackjack() const {
	int amount{ 0 };
	if (m_hand[0]->GetNumber() == 1 && m_hand[1]->GetNumber() >= 10) return true;
	else if (m_hand[1]->GetNumber() == 1 && m_hand[0]->GetNumber() >= 10) return true;
	return false;
}

/// <summary>
/// Dealer draws a card from the deck
/// this card is removed from the deck
/// </summary>
/// <param name="deck">From which the card is drawn</param>
/// <returns>Pointer to a card that was drawn</returns>
Card* Dealer::DrawCard(Deck& deck) {
	Card* card = new Card(*deck.back());
	return card;
}

/// <summary>
/// Add a card to the dealers hand
/// </summary>
/// <param name="card"></param>
void Dealer::AddCard(Card* card) {
	try {
		if (m_hand[0] == nullptr) m_hand[0] = card;
		else if (m_hand[1] == nullptr) m_hand[1] = card;
		else throw(m_name);
	}
	catch (std::string name) {
		std::cout << "CAN'T ADD A CARD TO DEALER " << name << "'s HAND SINCE IT'S FULL!";
	}
}

/// <summary>
/// Prints out the dealers hand
/// </summary>
void Dealer::ShowHand() const {
	std::cout << "Dealer " << m_name << "'s cards are : " << std::endl; m_hand[0]->ToString(); m_hand[1]->ToString();
}

/// <summary>
/// Prints out the dealers other card
/// </summary>
void Dealer::ShowCard() const {
	std::cout << "Dealer " << m_name << "'s other card is: "; m_hand[0]->ToString();
}

/// <summary>
/// Dealer deals cards from the deck to the players and itself
/// </summary>
/// <param name="deck">deck from which the cards are drawn from</param>
/// <param name="players">list of players that the cards are delt to</param>
void Dealer::DealCards(Deck& deck, std::vector<Player*> players) {
	for (int y = 0; y < 2; y++) {
		for (int i = 0; i < players.size(); i++) {
			Card* card = DrawCard(deck);
			players[i]->AddCard(card);
		}
		Card* card = DrawCard(deck);
		AddCard(card);
	}
}


