#include "Deck.h"
#include <algorithm>    // std::random_shuffle

// Constructor which creates all the cards for the deck
Deck::Deck() {
	for (int y = 0; y < types.size(); y++) {
		for (int i = 0; i < 13; i++) {
			Card* card = new Card(types[y], i + 1);
			cards.push_back(card);
		}
	}
}

/// <summary>
/// Copy constructor
/// </summary>
/// <param name="deck"></param>
Deck::Deck(const Deck& deck) {
	cards = deck.cards;
}

// Destructor
Deck::~Deck() {
	for (Card* card : cards) {
		delete card;
	}
	std::cout << "Deck destroyed" << std::endl;
}

/// <summary>
/// Deck is shuffled into a random order
/// </summary>
void Deck::Shuffle() {
	srand(time(0)); // This is called so the random generator gets a random value, since in c++ it isn't truly random
	std::random_shuffle(cards.begin(), cards.end());
}

/// <summary>
/// Prints out the decks cards in order
/// </summary>
void Deck::ToString() const {
	for (int i = 0; i < cards.size(); i++) {
		std::cout << i+1  << ": ";
		cards[i]->ToString();
	}
}

/// <summary>
/// Get the last card of the deck and remove it from the deck
/// </summary>
/// <returns>Last card of the deck</returns>
Card* Deck::back() {
	Card* card = cards.back();
	cards.pop_back();
	return card;
}